//********************************************************
//
// Example Application of BRD_xxx functions for PIO service.
//
// (C) InSys by Dorokhin Andrey Oct, 2007
//
//=********************************************************

#include	<windows.h>
#include	<stdio.h>
#include	<conio.h>
#include	<fcntl.h>
#include	<sys\stat.h>
#include	<share.h>
#include	<io.h>

#include	"brd.h"
#include	"ctrlpio.h"

const U16 PLD_CFG_TAG	= 0x0500; // tag of PLD

void GetOptions();
BRD_Handle GetSrvHandle(BRD_Handle handle, BRD_ServList* srv);

TCHAR g_pldFileName[MAX_PATH];
int g_isPldLoadAlways = 0;
int g_IoDir = 1;
int g_ReadMode = 1;
int g_ReadStrobe = 4;
int g_WriteStrobe = 4;

BRD_Handle g_hPio = 0;

S32 PioWrite();
S32 PioRead();

char iniFileName[] = "\\exam_pio.ini";

//=************************* main *************************
//=********************************************************
void main( int argc, char *argv[] )
{
	S32		status;

	BRD_displayMode(BRDdm_VISIBLE | BRDdm_CONSOLE); //     :     
	
	S32	DevNum;
	status = BRD_init("brd.ini", &DevNum); //  
//	BRD_Error* pErrInfo = NULL;
//	status = BRD_error(&pErrInfo); //    

	//   LID (   )
	BRD_LidList lidList;
	lidList.item = 10;
	lidList.pLID = new U32[10];
	status = BRD_lidList(lidList.pLID, lidList.item, &lidList.itemReal);

	GetOptions();

	BRD_Info	info;
	info.size = sizeof(info);
	BRD_Handle handle[10];

	for(ULONG iDev = 0; iDev < lidList.itemReal; iDev++)
	{
		BRD_getInfo(lidList.pLID[iDev], &info); //    
		printf("%s, DevID = 0x%x, RevID = 0x%x, Bus = %d, Dev = %d, Slot = %d, PID = %d.\n", 
			info.name, info.boardType >> 16, info.boardType & 0xff, info.bus, info.dev, info.slot, info.pid);
//		printf("Board: %s, DevID = 0x%x, RevID = 0x%x, BusType = %d, BaseMem = %x.\n", 
//			info.name, info.boardType >> 16, info.boardType & 0xff, info.busType, info.base[0]);
		//handle[iDev] = BRD_open(lidList.pLID[iDev], BRDopen_EXCLUSIVE, NULL); //     
		handle[iDev] = BRD_open(lidList.pLID[iDev], BRDopen_SHARED, NULL); //     
		if(handle[iDev] > 0)
		{
			BRD_PuList PuList[5];
			U32 ItemReal;
			status = BRD_puList(handle[iDev], PuList, 5, &ItemReal); //    
			if(ItemReal <= 5)
			{
				for(U32 j = 0; j < ItemReal; j++)
				{
					if(PuList[j].puCode == PLD_CFG_TAG)
					{ //    - 
						printf("PU%d: %s, Id = %d, Code = %x, Attr = %x.\n",
										j, PuList[j].puDescription, PuList[j].puId, PuList[j].puCode, PuList[j].puAttr);
						//   
						U32	PldState;
						status = BRD_puState(handle[iDev], PuList[j].puId, &PldState);
						printf("BRD_puState: PldState = %d\n", PldState);
						if(PuList[j].puId == 0x100)
						{
							if(g_isPldLoadAlways || !PldState)
							{ //   ADM
								printf("BRD_puLoad: loading...\r");
								status = BRD_puLoad(handle[iDev], PuList[j].puId, g_pldFileName, &PldState);
								if(BRD_errcmp(status, BRDerr_OK))
									printf("BRD_puLoad: load is OK\n");
								else
									if(BRD_errcmp(status, BRDerr_PLD_TEST_DATA_ERROR))
										printf("BRD_puLoad: data error starting test\n");
									else
										if(BRD_errcmp(status, BRDerr_PLD_TEST_ADDRESS_ERROR))
											printf("BRD_puLoad: address error starting test\n");
										else
											printf("BRD_puLoad: error loading\n");
							}
						}
					}
				}
			}
			BRD_ServList srvList[10];
			status = BRD_serviceList(handle[iDev], 0, srvList, 10, &ItemReal);
			if(ItemReal <= 10)
			{
				for(U32 j = 0; j < ItemReal; j++)
				{
					printf("Service %d: %s, Attr = %X.\n",
									j, srvList[j].name, srvList[j].attr);
				}
			}
			U32 iSrv;
			for(iSrv = 0; iSrv < ItemReal; iSrv++)
			{
				if(!strcmp(srvList[iSrv].name, "PIOSTD0"))
				{
					g_hPio = GetSrvHandle(handle[iDev], &srvList[iSrv]);
					PioWrite();
				}
			}
			if(g_hPio)
				status = BRD_release(g_hPio, 0);
			status = BRD_close(handle[iDev]); //   
			printf("\nBRD_close: OK\n");
		}
	}
	delete lidList.pLID;
	status = BRD_cleanup();
	printf("BRD_cleanup: OK\n");
	printf("Press any key for leaving program...\n");
	_getch();
}

void DisplayError(S32 status, TCHAR* cmd_str)
{
	S32	real_status = BRD_errext(status);
	TCHAR msg[255];
	switch(real_status)
	{
	case BRDerr_OK:
		sprintf_s(msg, "%s: BRDerr_OK\n", cmd_str);
		break;
	case BRDerr_BAD_MODE:
		sprintf_s(msg, "%s: BRDerr_BAD_MODE\n", cmd_str);
		break;
	case BRDerr_INSUFFICIENT_SERVICES:
		sprintf_s(msg, "%s: BRDerr_INSUFFICIENT_SERVICES\n", cmd_str);
		break;
	default:
		sprintf_s(msg, "%s: Unknown error\n", cmd_str);
		break;
	}
	printf(msg);
}

void GetOptions()
{
	TCHAR Buffer[128];
	TCHAR iniFilePath[MAX_PATH];
	GetCurrentDirectory(sizeof(iniFilePath)/sizeof(TCHAR), iniFilePath);
	lstrcat(iniFilePath, iniFileName);
	GetPrivateProfileString("Option", "PldFileName", "ambpcd_v10_admddc5016.mcs", g_pldFileName, sizeof(g_pldFileName), iniFilePath);
	GetPrivateProfileString("Option", "isPldLoadAlways", "0", Buffer, sizeof(Buffer), iniFilePath);
	g_isPldLoadAlways = atoi(Buffer);
	GetPrivateProfileString("Option", "IoDir", "1", Buffer, sizeof(Buffer), iniFilePath);
	g_IoDir = atoi(Buffer);
	GetPrivateProfileString("Option", "ReadMode", "1", Buffer, sizeof(Buffer), iniFilePath);
	g_ReadMode = atoi(Buffer);
	GetPrivateProfileString("Option", "ReadStrobe", "4", Buffer, sizeof(Buffer), iniFilePath);
	g_ReadStrobe = atoi(Buffer);
	GetPrivateProfileString("Option", "WriteStrobe", "4", Buffer, sizeof(Buffer), iniFilePath);
	g_WriteStrobe = atoi(Buffer);
}

BRD_Handle GetSrvHandle(BRD_Handle handle, BRD_ServList* srv)
{
	U32 mode = BRDcapt_EXCLUSIVE;
	BRD_Handle hSrv = BRD_capture(handle, 0, &mode, srv->name, 10000);
	if(mode == BRDcapt_EXCLUSIVE) printf("%s Capture mode: EXCLUSIVE\n", srv->name);
	if(mode == BRDcapt_SPY)	printf("%s Capture mode: SPY\n", srv->name);
	if(hSrv <= 0)
		printf("Error %s Capture\n", srv->name);
	return hSrv;
}

S32 PioWrite()
{
	S32		status;
	U32 type;
	status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_ISLVDS, &type);
	if(BRD_errcmp(status, BRDerr_OK))
	{
		char buf_type[16];
		if(type)
			lstrcpy(buf_type, "LVDS");
		else
			lstrcpy(buf_type, "LVTTL");
		printf("BRDctrl_PIO_ISLVDS: type of PIO is %s\n", buf_type);
	}
	else
		DisplayError(status, "BRDctrl_PIO_ISLVDS");
	BRD_PioDir pio_dir;
	pio_dir.lbDir = g_IoDir & 0x1;
	pio_dir.hbDir = (g_IoDir >> 1) & 0x1;
	status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_SETDIR, &pio_dir);
	if(BRD_errcmp(status, BRDerr_OK))
	{
		char buf_lbd[4], buf_hbd[4];
		lstrcpy(buf_lbd, "rd");
		lstrcpy(buf_hbd, "rd");
		if(pio_dir.lbDir)
			lstrcpy(buf_lbd, "wr");
		if(pio_dir.hbDir)
			lstrcpy(buf_hbd, "wr");
		printf("BRDctrl_PIO_SETDIR: low byte diir = %d(%s), hi byte dir = %d(%s)\n", pio_dir.lbDir, buf_lbd, pio_dir.hbDir, buf_hbd);
	}
	else
		DisplayError(status, "BRDctrl_PIO_SETDIR");

	BRD_PioMode pio_mode;
	pio_mode.rdMode = g_ReadMode;
	pio_mode.wrMode = 0;
	pio_mode.strobeFlow = 0;
	pio_mode.ackFlow = 0;
	pio_mode.rdStrobe = g_ReadStrobe;
	pio_mode.wrStrobe = g_WriteStrobe;
	status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_SETMODE, &pio_mode);
	if(BRD_errcmp(status, BRDerr_OK))
		printf("BRDctrl_PIO_SETMODE: pio_mode.rdMode = %d, ReaddStrobe = %d, WriteStrobe = %d\n",
													pio_mode.rdMode, pio_mode.rdStrobe, pio_mode.wrStrobe);
	else
		DisplayError(status, "BRDctrl_PIO_SETMODE");

//	U32 enbl = 1;
//	status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_ENABLE, &enbl);
	if(pio_dir.lbDir && pio_dir.hbDir)
	{ // pio 16-bit write
		printf("Write operation\n");
		U32 pio_data = 0;
		U32 pio_data0 = 0x5A5A;
		U32 pio_data1 = 0xA5A5;
		int fl_break = 0;
		printf("Hit any key to abort...\n");
		while(!fl_break & !_kbhit())
		{
			status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_WRITEDATA, &pio_data0);
			if(!BRD_errcmp(status, BRDerr_OK))
			{
				DisplayError(status, "BRDctrl_PIO_WRITEDATA");
				fl_break = 1;
			}
			status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_READDATA, &pio_data);
			pio_data &= 0xffff;
			printf("read pio_data = 0x%X\r", pio_data);
			Sleep(1);
			U32 pio_data = 0;
			status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_WRITEDATA, &pio_data1);
			if(!BRD_errcmp(status, BRDerr_OK))
			{
				DisplayError(status, "BRDctrl_PIO_WRITEDATA");
				fl_break = 1;
			}
			status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_READDATA, &pio_data);
			pio_data &= 0xffff;
			printf("read pio_data = 0x%X\r", pio_data);
			Sleep(1);
		}
		if(!fl_break)
			_getch();
	}
	else
		if(!pio_dir.lbDir && !pio_dir.hbDir)
		{ // pio 16-bit read
			printf("Read operation\n");
			U32 pio_data = 0;
			//U32 pio_data0 = 0x5A;
			//U32 pio_data1 = 0xA5;
			int fl_break = 0;
			printf("Hit any key to abort...\n");
			while(!fl_break & !_kbhit())
			{
				status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_READDATA, &pio_data);
				if(!BRD_errcmp(status, BRDerr_OK))
				{
					DisplayError(status, "BRDctrl_PIO_READDATA");
					fl_break = 1;
				}
				pio_data &= 0xffff;
				printf("read pio_data = 0x%X\r", pio_data);
				Sleep(1);
			}
			if(!fl_break)
				_getch();
		}
		else
		{
			printf("Write/Read operation\n");
			U32 rd_data = 0;
			U32 wr_data = 0;
			//if(pio_dir.lbDir)
			//	wr_data = 0x005A;
			//else
			//	wr_data = 0x5A00;
			int err_cnt = 0;
//			int fl_break = 0;
//			printf("Hit any key to abort...\n");
//			while(!fl_break & !_kbhit())
			for(int i = 0; i < 256; i++)
			{
				wr_data = i;
				if(pio_dir.hbDir)
					wr_data = wr_data << 8;
				status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_WRITEDATA, &wr_data);
				Sleep(2);
				status = BRD_ctrl(g_hPio, 0, BRDctrl_PIO_READDATA, &rd_data);
				if(pio_dir.lbDir)
					rd_data = (rd_data >> 8) & 0xff;
				else
					rd_data = rd_data & 0xff;
				rd_data &= 0xffff;
				if(rd_data != i)
					err_cnt++;
				printf("write pio_data = 0x%X  read pio_data = 0x%X  error count = %d\r", i, rd_data, err_cnt);
				//wr_data++;
			}
			//if(!fl_break)
			//	_getch();

		}
	return status;
}

//
// End of file
//
